xquery version "3.1";

module namespace lesapi="http://sepia.io/esapi/legacy-api";

declare namespace http="http://expath.org/ns/http-client";

(:
 : API that mirrors the current implementation used by Architrave and KMG
 : written by Moritz Schepp
 :)

declare function lesapi:search($service as xs:string, $index, $query) {
  let $json := local:to_json($query)
  let $response := http:send-request(
      <http:request method="POST" href="{$service}/{$index}/_search">
          <http:body media-type="application/json" method="text">{$json}</http:body>
      </http:request>
  )
  let $str := util:base64-decode($response[2])
  let $data := parse-json($str)
  return $data
};

declare function lesapi:drop-index($service as xs:string, $index) {
    let $response := http:send-request(
        <http:request method="DELETE" href="{$service}/{$index}" />
    )
    return $response
};

declare function lesapi:create-index($service as xs:string, $index, $data) {
    let $json := local:to_json($data)
    let $response := http:send-request(
        <http:request method="PUT" href="{$service}/{$index}">
            <http:body media-type="application/json" method="text">{$json}</http:body>
        </http:request>
    )
    let $ok := local:require_ok($response)
    return $response
};

declare function lesapi:index-doc($service as xs:string, $index, $doc) {
    let $json := local:to_json($doc)
    let $response := http:send-request(
        <http:request method="POST" href="{$service}/{$index}/_doc">
            <http:body media-type="application/json" method="text">{$json}</http:body>
        </http:request>
    )
    let $ok := local:require_ok($response)
    return $response
};

declare function lesapi:index-doc($service as xs:string, $index, $id, $doc) {
    let $json := local:to_json($doc)
    let $response := http:send-request(
        <http:request method="POST" href="{$service}/{$index}/_doc/{$id}">
            <http:body media-type="application/json" method="text">{$json}</http:body>
        </http:request>
    )
    let $ok := local:require_ok($response)
    return $response
};

declare function lesapi:refresh($service as xs:string) {
    let $response := http:send-request(
        <http:request method="GET" href="{$service}/_refresh" />
    )
    let $ok := local:require_ok($response)
    return $response
};

declare function local:to_json($data) {
  let $result := fn:serialize($data, map { 'method': 'json'})
  return $result
};

declare function local:require_ok($response) {
    let $code := xs:integer($response[1]/@status)
    return
        if ($code < 200 or $code > 299) then
            let $body := util:base64-decode($response[2])
            let $name := QName('http://datypic.com/err', 'ProdNumReq')
            let $msg := concat("(", string($code), ") elasticsearch responded with:", $body)
            return fn:error($name, $msg)
        else
            fn:true()
};
